require 'spec_helper'

class FakeController
  class << self
    attr_accessor :before_filters, :hidden_actions
  end

  attr_reader :km

  def initialize(km)
    @km = km
  end

  def self.before_filter(filter)
    @before_filters ||= []
    @before_filters << filter
  end

  def self.hide_action(hidden_actions)
    @hidden_actions ||= []
    @hidden_actions << hidden_actions
  end
end

describe Saucy::Kiss::ControllerFilters do
  let(:km) { stub('kissmetrics client', :identify => true) }
  let(:account) { stub('account', :id => 12345) }

  let(:controller_class) do
    Class.new(FakeController) do
      include Saucy::Kiss::ControllerFilters
    end
  end

  let(:controller) { controller_class.new(km) }

  it "includes #identify_to_kissmetrics as a before_filter" do
    controller_class.before_filters.should == [:identify_to_kissmetrics]
  end

  it "hides the #identify_to_kissmetrics method" do
    controller_class.hidden_actions.should == [:identify_to_kissmetrics]
  end

  it "sets identity based on the account if you're signed in and have a current_account" do
    controller.stub(:signed_out? => false,
                    :current_account? => true,
                    :current_account => account)
    km.should_receive(:identify).with("account-12345")
    controller.identify_to_kissmetrics
  end

  it "clears identity when you sign out" do
    controller.stub(:signed_out? => true)
    km.should_receive(:identify).with(nil)
    controller.identify_to_kissmetrics
  end

  it "does not re-identify if you're signed in but have no account, e.g. on the /accounts page" do
    controller.stub(:signed_out? => false,
                    :current_account? => false)
    km.should_receive(:identify).never()
    controller.identify_to_kissmetrics
  end
end
